from __future__ import print_function

## Title: robotSim.py
## Abstract: This program will simulate the Parallax Boe-Bot in the event that
##           you are unable to have access to a physical Boe-Bot.
## Author(s): Greg Greenleaf (pacohojaverde@gmail.com)
##            Alex Hauser (ahauser@csumb.edu)
##            Zac Leids (zleids@csumb.edu)
## Date: December 2, 2014

import socket, sys, pygame, time, os
if (sys.version_info < (3,0)):
    import thread
else: import _thread as thread
pygame.init()
pygame.display.set_caption('Computer C: BoeBot Simulator')

ip = "192.168.173.1" #Use this IP/port when testing over Ethernet
port = 55556

#Initialize a socket
client_socketA = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
client_socketA.connect((ip, port))

#Initialize PyGame variables for the GUI
size = width, height = 640, 480
speed = [2, 2]
black = 0, 0, 0
BLUE  = (  0,   0, 255)
screen = pygame.display.set_mode(size)
basicfont = pygame.font.SysFont(None, 48)
screenChanged = True

message = "Waiting for Messages from the Server..."

adjX = 0
adjY = 0

player = pygame.image.load(os.path.join("BoeBot.png"))
player.convert()
player = pygame.transform.scale(player, (100,100))

changeCount = 0

#use global variables to ensure the threads have access to them
global inp, End, Response

#end conditions
End_All = 0
End = 0

#Handles drawing new frames of the GUI
def Draw():
    global screenChanged
    global message
    global adjX, adjY
    text = basicfont.render(message, True, (255, 0, 0), (255, 255, 255))
    textrect = text.get_rect()
    textrect.centerx = screen.get_rect().centerx
    textrect.centery = 25

    screen.fill(black)

    #pygame.draw.line(screen, BLUE, (100, 100), (100, 150), 10)
    #pygame.draw.circle(screen, BLUE, (screen.get_rect().centerx + (adjX * 5), screen.get_rect().centery - 20 + (adjY * 5)), 15)
    screen.blit(player, (screen.get_rect().centerx - 50 + (adjX * 5), screen.get_rect().centery - 50 + (adjY * 5)))
    
   
    screen.blit(text, textrect)

    pygame.display.flip()
    #screenChanged = False
    return

# Handles reading and interpreting the commands being sent over
def parse(response):
    global message, screenChanged, adjX, adjY, acks
    forward = backward = left = right = 0
    screenChanged = True

    # A little input error detection
    if (len(str(response)) <= 20):
        for command in response:
            if (command == "0"): forward += 1
            elif (command == "1"): backward += 1
            elif (command == "2"): left += 1
            elif (command == "3"): right += 1

        print("\nCommand number " + str(acks) + ": " + str(response))
        print("================" + \
              "\nForward  x" + str(forward) + \
              "\nBackward x" + str(backward) + \
              "\nLeft     x" + str(left) + \
              "\nRight    x" + str(right) + \
              "\n================\n", end="\r")

        for command in response:
            if (command == "0"):
                message = "Forward"
                adjY -= 5
                if (screen.get_rect().centery - 20 + (adjY * 5) < 0):
                    adjY = 0
            elif (command == "1"):
                message = "Backward"
                adjY += 5
                if (screen.get_rect().centery - 20 + (adjY * 5) > 480):
                    adjY = 0
            elif (command == "2"):
                message = "Left"
                adjX -= 5
                if (screen.get_rect().centerx + (adjX * 5) < 0):
                    adjX = 0
            elif (command == "3"):
                message = "Right"
                adjX += 5
                if (screen.get_rect().centerx + (adjX * 5) > 640):
                    adjX = 0
            time.sleep(0.5)
    else: print("\nCommand exceeds max length...")
    return

# Handles receiving responses from the server
def receiver():
    global acks
    global screenChanged
    global changeCount
    acks = 0
    End = 0
    while End == 0:
            response = client_socketA.recv(1024).decode('utf-8')

            #If the response is non-null and non-empty,
            #print the response and set End to 1 to end the loop
            if str(response):
                screenChanged = True
                changeCount+=1
                #Acknowledging the server, encoding it with utf-8
                client_socketA.send(("ACK " + str(acks)).encode('utf-8'))
                acks += 1
                if response == "Close":
                    End = 1
                    client_socketA.close()
                    pygame.display.quit()
                    sys.exit()
                else:
                    thread.start_new_thread(parse, (response,))
                response = ""
    return

#start the threads
thread.start_new_thread(receiver, ())

while End == 0 and End_All == 0:
    if End != 0 or End_All != 0:
        thread.exit_thread()
        pygame.display.quit()
        print(End_All)
        client_socketA.close()
        sys.exit()
        break
    else:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                client_socketA.send("Goodbye".encode('utf-8'))
                thread.exit_thread()
                client_socketA.close()
                pygame.display.quit()
                sys.exit()

        if changeCount > 0:
            changeCount-=1
            screenChanged = True

        if screenChanged == True:
            Draw();

thread.exit_thread()
client_socketA.close()
sys.exit()
